/*******************************************************************************
* File Name    : LCD_LIB.c
* Version      : Rev. 1.00
* Device(s)    : R7F100GL
* Tool-Chain   : CCRL
* Description  : This file implements LCD control function via I2C bus
* Creation Date: 2021/02/01
* Modified Data: 2023/12/22 port to RL78/G23-FPB(IICA1)
********************************************************************************/

/*******************************************************************************
Includes
********************************************************************************/
#include "r_cg_macrodriver.h"
#include <Config_IICA0.h>
#include <Config_IICA1.h>
#include <Config_TAU0_5.h>
#include <Config_TAU0_6.h>
#include <Config_TAU0_7.h>
#include "LCD_LIB.h"
#include "r_cg_userdefine.h"


/*******************************************************************************
Pragma directive
********************************************************************************/


/*******************************************************************************
Global variables and functions
********************************************************************************/
volatile uint8_t  g_lcd_command[2];     // for set command for LCD
volatile uint8_t  g_lcd_data[2];        // for set data for LCD

extern volatile uint8_t sel_IICA;       // select IICA channel

void set_2digit( uint8_t data);             // display 2digit data
void set_1digit( uint8_t data);             // display 1digit data
uint8_t set_command( uint8_t lcd_command);  // set command to LCD

/*******************************************************************************
LCD display control functions for G23_FPB
********************************************************************************/

/*---------------------------------------------------------------------
* Function Name: init_LCD
* Description  : This function initialize LCD
* Arguments    : none
* Return Value : result
*                   0 : success
*                   2 : No_slave
*                   3 : No ACK for data
----------------------------------------------------------------------*/
 uint8_t init_LCD(void)
{

    uint8_t status;

    status =  set_command(CLRDISP);     // initialize LCD display
    if ( 0 != status ) return(2);

    delay(3);                           // wait for 3ms
    set_command(LCD_Mode);              // set LCD operation mode
    delayMicroseconds(60);              // wait for 60us
    set_command(DISPON);                // display on
    delayMicroseconds(60);              // wait for 60us
    set_command(ENTRY_Mode);            // set data entry mode
    delayMicroseconds(60);              // wait for 60us

    return(0);

}   /* end of init_LCD function   */


/*******************************************************************************
* Function Name: move_cursor
* Description  : This function move cursor position
* Arguments    : cursor address
*               col: 0 to 0x0F
*               row : 0(line1) /1(line2)
* Return Value : none
********************************************************************************/
void move_cursor( uint8_t col , uint8_t row)
{

    uint8_t work;

    work = (( row & 1 ) * 0x40);        // set top of line address
    work += ( col & 0x0F );             // add character position
    work |= 0x80;                       // get cursor move command

    set_command( work );                // set curso move command
    delayMicroseconds(120);             // wait for 120us

}   /* end of move_cursor function  */


/*******************************************************************************
* Function Name: print_LCD
* Description  : This function print 16chars 2lines
* Arguments    : data
* Return Value : none
********************************************************************************/
void print_LCD( uint8_t *point, uint8_t *point2)
{

    uint8_t data;
    uint8_t work;
    uint8_t *p_addr;

    p_addr = point;                         // set pointer
    move_cursor(0,0);                       // move cursor top of 1st line
    for ( work = 0 ; work < 16 ; ++work )
    {
        data = *p_addr;                     // read data
        if ( 0 == data )
        {
            data = 0x20;                    // change null data to space code
        }
        set_data( data );                   // send data 
        ++p_addr;
        delayMicroseconds(60);              // wait for 60us

    }

    p_addr = point2;                        // set pointer
    move_cursor(0,1);                       // move cursor top of 2nd line
    for ( work = 0 ; work < 16 ; ++work )
    {
        data = *p_addr;                     // read data
        if ( 0 == data )
        {
            data = 0x20;                    // change null data to space code
        }
        set_data( data );                   // send data 
        ++p_addr;
        delayMicroseconds(60);              // wait for 60us

    }

    p_addr = point;                         // set pointer
    move_cursor(0,0);                       // move cursor top of 1st line
    for ( work = 0 ; work < 16 ; ++work )
    {
        data = *p_addr;                     // read data
        if ( 0 == data )
        {
            data = 0x20;                    // change null data to space code
        }
        set_data( data );                   // send data 
        ++p_addr;
        delayMicroseconds(60);              // wait for 60us

    }

}   /* end of print_LCD function    */


/*******************************************************************************
* Function Name: set_2digitBCD
* Description  : This function display 2 digits
* Arguments    : display number(99-0)
* Return Value : none
********************************************************************************/
void set_2digitBCD( uint8_t data)
{

    uint8_t work;

    work = data;                        // get display data
    
    set_1digit((work / 10));            // display digit of 10
    delayMicroseconds(60);              // wait for 60us
    set_1digit((work % 10));            // display digit of 1
    delayMicroseconds(60);              // wait for 60us

}   /* end of set_2digitBCD function    */



/*******************************************************************************
* Function Name: set_2digit
* Description  : This function display 2 digits
* Arguments    : display number(packed BCD or HEX)
* Return Value : none
********************************************************************************/
void set_2digit( uint8_t data)
{

    uint8_t work;

    work = data;                        // get display data

    set_1digit( work >> 4 );            // display higher 4bit
    delayMicroseconds(60);              // wait for 60us
    set_1digit( data );                 // display lowwer 4bit
    delayMicroseconds(60);              // wait for 60us

}   /* end of set_2digit function   */


/*******************************************************************************
* Function Name: set_1digit
* Description  : This function display 1 digit(lowwer 4bit of data)
* Arguments    : display lowwer 4bit as ASCII
* Return Value : none
********************************************************************************/
void set_1digit( uint8_t data)
{

    uint8_t work;

    work = (data & 0x0F);               // mask unused bit
    if ( work < 0x0A )
    {
        work += 0x30;                   // convert 0-9
    }
    else
    {
        work += 0x37;                   // convert A-F
    }

    set_data(work);                     // display 1digit

}   /* end of set_1digit function   */


/*---------------------------------------------------------------------
* Function Name: set_command
* Description  : This function set command to LCD
* Arguments    : command
* Return Value : result
*                   0x00(SUCCESS)   : success(operation complete)
*                   0x8F(BUS_ERROR) : bus or IICAnis busy error
*                   0x80(NO_SLAVE)  : NACK for slave address
*                   0xC0(NO_ACK)    : NACK for transmit data
----------------------------------------------------------------------*/
uint8_t set_command( uint8_t lcd_command)
{

    uint8_t result;                     // result of transmission

    g_lcd_command[0] = COMBYTE;         // set command of LCD
    g_lcd_command[1] = lcd_command;     // set instruction for LCD

    if ( 0 == sel_IICA )                // used IIC channel is 0
    {
        result = R_Config_IICA0_Master_Send (
            SLADDR_LCD*2,                   // slave address
            (uint8_t *__near)g_lcd_command, // data buffer address
            2,                              // transmit data number
            0xFF);                          // time out is max

        if ( 0x00 == result )
        {
            result = R_IICA0_wait_comend(1);// wait for transmission end
        }
    }
    else
    {                                   // used IIC channel is 1
        result = R_Config_IICA1_Master_Send (
            SLADDR_LCD*2,                   // slave address
            (uint8_t *__near)g_lcd_command, // data buffer address
            2,                              // transmit data number
            0xFF);                          // time out is max

        if ( 0x00 == result )
        {
            result = R_IICA1_wait_comend(1);// wait for transmission end
        }
    }
    return(result);

}   /* end of set_command function  */



/*******************************************************************************
* Function Name: set_data
* Description  : This function send data to LCD
* Arguments    : data code
* Return Value : result
*                   0x00(SUCCESS)   : success(operation complete)
*                   0x8F(BUS_ERROR) : bus or IICAn is busy error
*                   0x80(NO_SLAVE)  : NACK for slave address
*                   0xC0(NO_ACK)    : NACK for transmit data
********************************************************************************/
uint8_t set_data(uint8_t datacode)
{

    unsigned char result;               // result of transmission

    g_lcd_data[0] = DATABYTE;           // set command for LCD
    g_lcd_data[1] = datacode;           // set data for LCD

    if ( 0 == sel_IICA )                // used IIC channel is 0
    {
        result = R_Config_IICA0_Master_Send (
            SLADDR_LCD*2,                   // slave address
            (uint8_t *__near)g_lcd_data,    // data buffer address
            2,                              // transmit data number
            0xFF);                          // time out is max

        if ( 0x00 == result )
        {
            result = R_IICA0_wait_comend(1);// wait for transmission end
        }

    }
    else
    {                                  // used IIC channel is 1
        result = R_Config_IICA1_Master_Send (
            SLADDR_LCD*2,                   // slave address
            (uint8_t *__near)g_lcd_data,    // data buffer address
            2,                              // transmit data number
            0xFF);                          // time out is max

        if ( 0x00 == result )
        {
            result = R_IICA1_wait_comend(1);// wait for transmission end
        }
    }

    return(result);

}   /* end of set_data function */
